#!/usr/bin/env python3
"""
Network Configuration Backup Tool
Parameters:
    - IP: Device IP address
    - port: SSH port number
Additional parameters in comments:
    - username: SSH username
    - password: SSH password
    - device_type: Device type (cisco, juniper, etc.)
"""
import paramiko
import argparse
import time
from datetime import datetime
import logging
from pathlib import Path
import json
import re
import threading
import queue
import getpass

class NetworkConfigBackup:
    def __init__(self, devices, backup_dir="network_backups"):
        self.devices = devices
        self.backup_dir = Path(backup_dir)
        self.backup_dir.mkdir(exist_ok=True)
        self.results_queue = queue.Queue()
        self.setup_logging()
        
    def setup_logging(self):
        """Set up logging configuration"""
        logging.basicConfig(
            level=logging.INFO,
            format='%(asctime)s - %(levelname)s - %(message)s',
            handlers=[
                logging.FileHandler(self.backup_dir / "backup.log"),
                logging.StreamHandler()
            ]
        )
        
    def get_device_commands(self, device_type):
        """Get commands for specific device type"""
        commands = {
            'cisco': {
                'show_run': 'show running-config',
                'show_version': 'show version',
                'show_inventory': 'show inventory',
                'show_interfaces': 'show interfaces',
                'show_ip_route': 'show ip route'
            },
            'juniper': {
                'show_config': 'show configuration | display set',
                'show_version': 'show version',
                'show_interfaces': 'show interfaces',
                'show_route': 'show route'
            },
            'arista': {
                'show_run': 'show running-config',
                'show_version': 'show version',
                'show_inventory': 'show inventory',
                'show_interfaces': 'show interfaces'
            },
            'generic': {
                'show_run': 'show running-config',
                'show_version': 'show version'
            }
        }
        
        return commands.get(device_type.lower(), commands['generic'])
        
    def sanitize_output(self, output):
        """Remove sensitive information from config"""
        # Remove passwords
        output = re.sub(r'password \S+', 'password <removed>', output)
        output = re.sub(r'secret \S+', 'secret <removed>', output)
        
        # Remove SNMP community strings
        output = re.sub(r'community \S+', 'community <removed>', output)
        
        # Remove keys
        output = re.sub(r'key \S+', 'key <removed>', output)
        
        return output
        
    def backup_device(self, device):
        """Backup single device configuration"""
        try:
            # Create SSH client
            ssh = paramiko.SSHClient()
            ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
            
            # Connect to device
            logging.info(f"Connecting to {device['ip']}...")
            ssh.connect(
                device['ip'],
                port=device.get('port', 22),
                username=device['username'],
                password=device['password'],
                timeout=10
            )
            
            # Get device commands
            commands = self.get_device_commands(device.get('type', 'generic'))
            
            # Create device backup directory
            device_dir = self.backup_dir / f"{device['ip']}_{datetime.now().strftime('%Y%m%d_%H%M%S')}"
            device_dir.mkdir(exist_ok=True)
            
            results = {
                'device': device['ip'],
                'timestamp': datetime.now().isoformat(),
                'status': 'success',
                'files': []
            }
            
            # Execute commands and save output
            for cmd_name, command in commands.items():
                try:
                    stdin, stdout, stderr = ssh.exec_command(command, timeout=30)
                    output = stdout.read().decode()
                    
                    if output:
                        # Sanitize output
                        safe_output = self.sanitize_output(output)
                        
                        # Save to file
                        filename = device_dir / f"{cmd_name}.txt"
                        with open(filename, 'w') as f:
                            f.write(safe_output)
                            
                        results['files'].append({
                            'command': command,
                            'filename': str(filename),
                            'size': len(safe_output)
                        })
                        
                except Exception as e:
                    logging.error(f"Error executing {command} on {device['ip']}: {str(e)}")
                    results['files'].append({
                        'command': command,
                        'error': str(e)
                    })
                    
            ssh.close()
            
        except Exception as e:
            logging.error(f"Error backing up {device['ip']}: {str(e)}")
            results = {
                'device': device['ip'],
                'timestamp': datetime.now().isoformat(),
                'status': 'error',
                'error': str(e)
            }
            
        self.results_queue.put(results)
        
    def backup_all_devices(self):
        """Backup all devices in parallel"""
        logging.info(f"Starting backup of {len(self.devices)} devices")
        
        threads = []
        for device in self.devices:
            thread = threading.Thread(target=self.backup_device, args=(device,))
            thread.start()
            threads.append(thread)
            
        # Wait for all backups to complete
        for thread in threads:
            thread.join()
            
        # Collect results
        results = []
        while not self.results_queue.empty():
            results.append(self.results_queue.get())
            
        # Save summary report
        self.save_summary(results)
        
    def save_summary(self, results):
        """Save backup summary report"""
        summary = {
            'timestamp': datetime.now().isoformat(),
            'total_devices': len(self.devices),
            'successful_backups': len([r for r in results if r['status'] == 'success']),
            'failed_backups': len([r for r in results if r['status'] == 'error']),
            'results': results
        }
        
        summary_file = self.backup_dir / f"summary_{datetime.now().strftime('%Y%m%d_%H%M%S')}.json"
        with open(summary_file, 'w') as f:
            json.dump(summary, f, indent=2)
            
        logging.info(f"\nBackup complete. Summary saved to {summary_file}")
        logging.info(f"Successful backups: {summary['successful_backups']}")
        logging.info(f"Failed backups: {summary['failed_backups']}")
        
    def verify_backups(self):
        """Verify backup files and their integrity"""
        logging.info("Verifying backups...")
        
        for device_dir in self.backup_dir.glob("*_*"):
            if device_dir.is_dir():
                files = list(device_dir.glob("*.txt"))
                if not files:
                    logging.warning(f"No backup files found in {device_dir}")
                    continue
                    
                for file in files:
                    try:
                        with open(file, 'r') as f:
                            content = f.read()
                            if not content.strip():
                                logging.warning(f"Empty backup file: {file}")
                            elif len(content) < 100:
                                logging.warning(f"Suspiciously small backup file: {file}")
                    except Exception as e:
                        logging.error(f"Error verifying {file}: {str(e)}")

def load_devices_from_file(filename):
    """Load device list from JSON file"""
    with open(filename, 'r') as f:
        return json.load(f)

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Network Configuration Backup Tool")
    parser.add_argument("--devices", help="JSON file containing device list")
    parser.add_argument("--backup-dir", default="network_backups",
                      help="Directory to store backups")
    parser.add_argument("--interactive", action="store_true",
                      help="Interactive mode for single device backup")
    args = parser.parse_args()
    
    if args.interactive:
        # Interactive mode
        print("Enter device details:")
        devices = [{
            'ip': input("IP address: "),
            'port': int(input("SSH port [22]: ") or "22"),
            'username': input("Username: "),
            'password': getpass.getpass("Password: "),
            'type': input("Device type (cisco/juniper/arista/generic) [generic]: ") or "generic"
        }]
    elif args.devices:
        # Load devices from file
        devices = load_devices_from_file(args.devices)
    else:
        parser.error("Either --devices or --interactive must be specified")
        
    backup = NetworkConfigBackup(devices, args.backup_dir)
    backup.backup_all_devices()
    backup.verify_backups()
